/***************************************************************************
 *            stp.h
 *
 *  Sat Jun 18 19:39:47 2005
 *  Copyright  2005  Carbone Francesco
 *  trini4mail@yahoo.it
 *	franci.carbone@gmail.com
 ****************************************************************************/



#include "stp_timer.h"
#include "stp_utility.h"

#include <stdio.h>

 
 #define False	0
 #define True	1

/* Port State Codified with a set of integer constants */
#define Disabled	0
#define Listening	1
#define	Learning	2
#define	Forwarding	3
#define Blocking	4

#define Zero 	0
#define One 	1


/*********** Tunnables Protocol Parameters *******************
 *													
 * The Bridge shall enforce the following relationship :	
 *													
 * 2x(BRIDGE_FORWARD_DELAY - 1.0) >= BRIDGE_MAX_AGE		
 *													
 * BRIDGE_MAX_AGE >= 2x(BRIDGE_HELLO_TIME + 1)			
 *													
 *************************************************************
 
 ******************Defaults Values***********************
 *	This values is used as defaults values , when the 
 *	user defined there aren't
 */

#define BRIDGE_DIAMETER 			7	// Recommended value 7
#define BRIDGE_TRANSIT_DELAY 		1 	// Recommended value 1.0 - 4.0
#define MESSAGE_AGE_INCREMENT 		1 	// Recommended value 1.0 - 4.0
#define BPDU_TRANSMISSION_DELAY 	1 	// Recommended value 1.0 - 4.0

#define BRIDGE_HELLO_TIME		2	// Recommended value 1.0 - 10.0
#define BRIDGE_MAX_AGE			20	// Recommended value 6.0 - 40.0

#define BRIDGE_FORWARD_DELAY 	15 	// Recommended value 4.0 - 30.0
#define HOLD_TIME				1	// Fixet to 1.0

#define TOPOLOGY_CHANGE_TIME	(BRIDGE_HELLO_TIME*2)

#define PATH_COST				10  

#define BRIDGE_PRIORITY		32768	// Recommended value 0 - 65535
#define PORT_PRIORITY		128		// Recommended value 0 - 255

//**** Standard BPDU parameters 	****
//****	 used in BPDU fields 	****
#define BPDU_PROTOCOL_IDENTIFIER 	0x0000
#define BPDU_PROTOCOL_VERSION		0x00


/* Bpdu type definitions */

/*	Configuration BPDU type codified in BPDU field */
#define CONFIG_BPDU_TYPE	0x00

//	Topology Change Notification BPDU codified in BPDU field
#define TCN_BPDU_TYPE		0x80


/*
 *	TOPOLOGY CHANGE BPDU type definition
 *	structure used to store Topology change BPDU fields
 */
struct stp_tcnbpdu	{	
	uint16_t	tcn_protoid;	// protocol identifier
	uint8_t		tcn_protover;	// protocol version
	uint8_t		tcn_bpdutype;	// bdpu type
};

/* 
 *	CONFIGURATION BPDU type definitions
 * 	structure used to store Configuration BPDU fields 
 */
struct	stp_configbpdu	{	
	uint16_t		cbpdu_protoid;	// protocol identifier
	uint8_t		cbpdu_protover;	// protocol version
	uint8_t		cbpdu_bpdutype;	// bpdu type	
	uint8_t		cbpdu_flags; 	//	topology Change Acknowledgement codified in bit 8 ;
									// 	topology Change codified in bit 1;
	uint64_t		cbpdu_designatedroot; 	// designated root identifier in the Bridged LAN
	uint32_t		cbpdu_rootpathcost;   	// root path cost of the link used by the port 
	uint64_t		cbpdu_bridgeid;			// bridge identifier 
	uint16_t		cbpdu_portid;			// port identifier
	uint16_t		cbpdu_messageage;		// message age
	uint16_t		cbpdu_maxage;			// max age is a timeout value of the message age timer
	uint16_t		cbpdu_hellotime;		// hello time is a timeout value of the hello timer 
	uint16_t		cbpdu_forwarddelaytime;	// forward delay time is a timeout value of the forward delay timer

};

/*
 *	Port Data type
 *	Structure used to store Port parameters and timers
 */
struct	stp_portdata	{
	uint16_t 			pd_index;			
	uint16_t			pd_portid;				// port identifier
	uint8_t				pd_state; 				// state ( disabled, blocking, forwarding, listening , learning)
	uint32_t			pd_pathcost;			// path cost of the link
	uint64_t			pd_designatedroot;		// designated root identifier for the lan to which the port is attached
	uint32_t			pd_designatedcost;		// designated cost offered from the designated port of the lan
	uint64_t			pd_designatedbridge;	// designated Bridge identifier for the lan to which the port is attached  
	uint16_t 			pd_designatedport;		// designated port identifier for the lan to which the port is attacched
	int					pd_topologychangeack;	// flag value indicating topology change acknowledgement ( true or false codified as 1 or 0 integer )
	int					pd_configpending;		// configuration pending indicate the necessity of transmitting a Configuration BPDU when old timer expire
	int					pd_changedetectionenabled;	// change detection enabled indicate when it is set to True ( integer value 1 ) 
													// the ability of port to detect a topology change
	struct stp_timer 	pd_timermessageage;			// message age timer
	struct stp_timer 	pd_timerforwarddelay;		// forward delay timer
	struct stp_timer	pd_timerhold;				// hold timer
};


/*
 *	Bridge info data type
 *	structures containig Bridge parameters, Port and Timers structures
 */

struct stp_bridgedata	{	
	// Root depending bridge parameters
	uint64_t 			bd_designatedroot;	// Designated Root Identifier that indicates the Root in Bridged LAN
	uint32_t			bd_rootpathcost;	// root path cost indicating the cost of path to the root
	uint16_t			bd_rootport;		// root port
	uint16_t			bd_maxage;			// max age, root timeout values
	uint16_t			bd_hellotime;		// hello time, root timeout values
	uint16_t			bd_forwarddelaytime; // forward delay time, root timeout values
	// Bridge own parameters
	uint64_t			bd_bridgeid;				// Bridge Identifier
	uint16_t			bd_bridgemaxage;			// Bridge max age
	uint16_t			bd_bridgehellotime;			// Bride hello time 
	uint16_t			bd_bridgeforwarddelaytime;	// Bridge forward delay time, bridge timeout values
	uint16_t			bd_topologychangetime;		// topology change time, Bridge timeout values
	uint16_t			bd_holdtime;				// hold time bridge timeout values
	// Bridge Flags
	int					bd_topologychangedetected;	// topology change detectedset to 1 if topology change was detected
	int					bd_topologychange;			// topology change
	// Bridge associated ports
	struct stp_portdata		**bd_ports;				// dinamic array provides pointers to ports structure
	uint16_t			bd_numofports;				// number of total ports +1
	// Bridge Timers
	struct stp_timer	bd_timertopologychange;		// topology change timer
	struct stp_timer	bd_timertcn;				// topology change notification timer
	struct stp_timer	bd_timerhello;				// hello timer
};
 

void 	stp_received_configuration_bpdu(struct stp_bridgedata *, uint16_t, struct stp_configbpdu );
void 	stp_received_tcn_bpdu(struct stp_bridgedata *, uint16_t);
void 	stp_tick(struct stp_bridgedata *);
struct 	stp_bridgedata *stp_create_bridge(const char *);

void	stp_initialization(struct stp_bridgedata *);
void	stp_initialize_port(struct stp_bridgedata *, uint16_t);
void 	stp_enable_port(struct stp_bridgedata *, uint16_t);
void 	stp_disable_port(struct stp_bridgedata *, uint16_t);
void 	stp_set_bridge_priority(struct stp_bridgedata *, uint16_t);
void 	stp_set_port_priority(struct stp_bridgedata *, uint16_t, uint16_t);
void 	stp_set_path_cost(struct stp_bridgedata *, uint16_t , uint32_t);
void 	stp_enable_change_detection(struct stp_bridgedata *, uint16_t );
void 	stp_disable_change_detection(struct stp_bridgedata *, uint16_t );
	// Remove code below after testing
int		stp_root_port(struct stp_bridgedata *,uint16_t );
int 	stp_designated_port(struct stp_bridgedata *,uint16_t );
