/***************************************************************************
 *            stp_utility.c
 *
 *  Wed Jun 15 18:14:51 2005
 *  Copyright  2005  trini
 *  trini4mail@yahoo.it
 *	franci.carbone@gmail.com
 ****************************************************************************/
 
 
 /*
  *	This code implements a set of functions to manipulate
  * Bridge Identifier, Port Identifier and BPDU fields
  *	
  */
#include "stp_utility.h"

// This constants are used to test a single bit
#define	SET		0x01
#define RESET	0x00

// Topology Change  Acknowledgment Flag ( bit 8 set to 1 )
#define FLAG_TCACK_SET 	0x80

// Topology Change Flag ( bit 1 of 8 set to 1 )
#define FLAG_TC_SET		0x01



/******* BRIDGE IDENTIFIER UTILITY*******/

/* 	
 *	Set the priority in the Bridge Identifier, coded in its last 16  
 *	high significant bits
 */
void 
stp_set_priority_bridgeid(uint64_t *rootId , uint16_t p)
{	
	(*rootId) <<= 16;
	(*rootId) >>= 16;
	(*rootId) |= (((uint64_t)p) << 48);
}

/*	
 *	Set the address of a Bridge Identifier , coded 
 *	in its first 48 lower significant bits
 */
void 
stp_set_address_bridgeid(uint64_t *rootId, uint64_t address)
{	(*rootId) >>= 48;
	(*rootId) <<= 48;
	(*rootId) |= ((uint64_t)address);
}
	

/* 	
 *	Return the address ( 48 bit coded address ) 
 *	from a Bridge Identifier 
 *
 */
uint64_t
stp_get_address_bridgeid(uint64_t bridgeId)
{	
	bridgeId <<= 16;
	return ((uint64_t)(bridgeId >> 16));
}

/*	
 *	Return priority ( 16 bit ) from a 
 *	Bridge Identifier  
 */
uint16_t 
stp_get_priority_bridgeid(uint64_t rootId)
{	
	rootId >>= 48;
	return ((uint16_t)(rootId));
}

/******** PORT IDENTIFIER UTILITY **********/

/*	
 *	Set the Priority ( high 8 bits ) in a Port Identifier
 *	( 16 bits )
 */
void stp_set_portno_portid(uint16_t *portId, uint16_t p)
{	
	(*portId) >>= 8;
	(*portId) <<= 8;
	(*portId) |= p;
}

// Set Port number in the 8 lower bits of a Port Identifier
void 
stp_set_priority_portid(uint16_t *portId, uint16_t nPort)
{	
	(*portId) <<= 8;
	(*portId) >>= 8;
	(*portId) |= ((uint16_t)nPort << 8);
}

// Return port number from a Port Identifier
uint16_t 
stp_get_portno_portid(uint16_t portId)
{	
	portId <<=8 ;
	portId >>=8 ;
	return (uint16_t)(portId);
}

// Return priority ( 8 bits ) from a port identifier
uint16_t 
stp_get_priority_portid(uint16_t portId)
{
	return ((uint16_t)(portId>>8));
}

/******* FLAG UTILITY ********/

// Set bit 8 to 1 in a 8 bit coded Flag
void 	
stp_set_tcack(uint8_t *flag)
{
	(*flag) = ((*flag) & FLAG_TC_SET);
	(*flag) = ((*flag) | FLAG_TCACK_SET);
}

// Set bit 8 to 0 in a 8 bit coded Flag
void 	
stp_reset_tcack(uint8_t *flag)
{	
	(*flag) = ((*flag) & FLAG_TC_SET);
}

// Set bit 1 to 1 in a 8 bit coded Flag	
void 	
stp_set_tc(uint8_t	*flag)
{	
	(*flag) = ((*flag) & FLAG_TCACK_SET);
	(*flag) = ((*flag) | FLAG_TC_SET);
}

// Set bit 1 to 0 in a 8 bit coded Falg
void	
stp_reset_tc(uint8_t *flag)
{	
	(*flag) = ((*flag) & FLAG_TCACK_SET);
}

// Test if topology change is set in a 8 bit coded Flag 
int	
stp_is_tc_set(uint8_t flag)
{	
	return ( (flag & FLAG_TC_SET) == FLAG_TC_SET); 
}

// Test if topology change is set in a 8 bit coded Flag
int	
stp_is_tcack_set(uint8_t flag)
{	
	return ( (flag & FLAG_TCACK_SET) == FLAG_TCACK_SET );	
}
